<?php

// increase timeout limit
set_time_limit(60);

require '../spotify/spotify.php';
require '../spotify/lib.php';

if (isset($_GET['logout'])) {
    $_SESSION['accessToken'] = '';
    $_SESSION['refreshToken'] = '';
}

$_SESSION['accessToken'] = isset($_SESSION['accessToken']) ? $_SESSION['accessToken'] : '';
$_SESSION['refreshToken'] = isset($_SESSION['refreshToken']) ? $_SESSION['refreshToken'] : '';

if (!$_SESSION['accessToken']) {
    header('Location: auth.php');
    die();
}

$session = new SpotifyWebAPI\Session($CLIENT_ID, $CLIENT_SECRET, $REDIRECT_URI);

// Use previously requested tokens fetched from session
if ($_SESSION['accessToken']) {
    $session->setAccessToken($_SESSION['accessToken']);
    $session->setRefreshToken($_SESSION['refreshToken']);
} else {
    // Or request a new access token
    $session->refreshAccessToken($_SESSION['refreshToken']);
}

$options = [
    'scope' => $SPOTIFY_REQUIRED_SCOPES,
    'auto_refresh' => true,
];

$api = new SpotifyWebAPI\SpotifyWebAPI($options, $session);

// Save new tokens, they might have been updated
$_SESSION['accessToken'] = $session->getAccessToken();
$_SESSION['refreshToken'] = $session->getRefreshToken();

// ################################################################

$user = [
    'id' => '',
    'image_url' => '',
    'display_name' => '',
    'profile_url' => '',
];
try {
    $me = $api->me();

    $user = [
        'id' => $me->id,
        'image_url' => \count($me->images) ? $me->images[0]->url : '',
        'display_name' => $me->display_name,
        'profile_url' => $me->external_urls->spotify,
    ];
} catch (Exception $ex) {
	error_log($ex->getMessage());
}
